@@ -8,6 +8,7 @@ from crispy_forms.bootstrap import (
     Alert,
     AppendedText,
     FieldWithButtons,
+    FormActions,
     InlineCheckboxes,
     InlineField,
     InlineRadios,
@@ -200,7 +201,6 @@ class TestBootstrapLayoutObjects:
         form.helper.layout = Layout("inline_radios")
 
         html = render_crispy_form(form)
-        print(html)
         assert 'class="form-check-input"' in html
 
         # Make sure an inherited CheckboxSelectMultiple gets rendered as it
@@ -246,6 +246,8 @@ class TestBootstrapLayoutObjects:
         form.helper.layout = Layout(InlineRadios("inline_radios"))
         if django.VERSION < (5, 0):
             expected = "inline_radios_failing_lt50.html"
+        elif django.VERSION < (5, 2):
+            expected = "inline_radios_failing_lt52.html"
         else:
             expected = "inline_radios_failing.html"
         assert parse_form(form) == parse_expected(expected)
@@ -256,6 +258,8 @@ class TestBootstrapLayoutObjects:
         form.helper.layout = InlineRadios("checkboxes")
         if django.VERSION < (5, 0):
             expected = "inline_checkboxes_failing_lt50.html"
+        elif django.VERSION < (5, 2):
+            expected = "inline_checkboxes_failing_lt52.html"
         else:
             expected = "inline_checkboxes_failing.html"
         assert parse_form(form) == parse_expected(expected)
@@ -274,6 +278,39 @@ class TestBootstrapLayoutObjects:
         )
         assert parse_form(form) == parse_expected("accordion.html")
 
+    def test_accordion_css_class_is_applied(self):
+        classes = "one two three"
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.form_tag = False
+        test_form.helper.layout = Layout(
+            Accordion(
+                AccordionGroup("one", "first_name"),
+                css_class=classes,
+                css_id="super-accordion",
+            )
+        )
+        html = render_crispy_form(test_form)
+
+        assert (
+            html.count('<div class="accordion %s" id="super-accordion"' % classes) == 1
+        )
+
+    def test_accordion_group_css_class_is_applied(self):
+        classes = "one two three"
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.form_tag = False
+        test_form.helper.layout = Layout(
+            Accordion(
+                AccordionGroup("one", "first_name"),
+                AccordionGroup("two", "password1", "password2", css_class=classes),
+            )
+        )
+        html = render_crispy_form(test_form)
+
+        assert html.count('<div class="accordion-item %s"' % classes) == 1
+
     def test_accordion_active_false_not_rendered(self):
         test_form = SampleForm()
         test_form.helper = FormHelper()
@@ -403,30 +440,13 @@ class TestBootstrapLayoutObjects:
                     "one",
                     "first_name",
                     css_id="custom-name",
-                    css_class="first-tab-class active",
+                    css_class="first-tab-class",
                 ),
                 Tab("two", "password1", "password2"),
+                css_class="tab-holder-class",
             )
         )
-        html = render_crispy_form(test_form)
-
-        assert (
-            html.count(
-                '<ul class="nav nav-tabs"> <li class="nav-item">'
-                '<a class="nav-link active" href="#custom-name" data-bs-toggle="tab">'
-                "One</a></li>"
-            )
-            == 1
-        )
-        assert html.count("tab-pane") == 2
-
-        assert html.count('class="tab-pane first-tab-class active"') == 1
-
-        assert html.count('<div id="custom-name"') == 1
-        assert html.count('<div id="two"') == 1
-        assert html.count('name="first_name"') == 1
-        assert html.count('name="password1"') == 1
-        assert html.count('name="password2"') == 1
+        assert parse_form(test_form) == parse_expected("test_tab_and_tab_holder.html")
 
     def test_tab_helper_reuse(self):
         # this is a proper form, according to the docs.
@@ -578,6 +598,8 @@ class TestBootstrapLayoutObjects:
         )
         if django.VERSION < (5, 0):
             expected = "test_floating_field_failing_lt50.html"
+        elif django.VERSION < (5, 2):
+            expected = "test_floating_field_failing_lt52.html"
         else:
             expected = "test_floating_field_failing.html"
         assert parse_form(form) == parse_expected(expected)
@@ -586,7 +608,11 @@ class TestBootstrapLayoutObjects:
         form = GroupedChoiceForm()
         form.helper = FormHelper()
         form.helper.layout = Layout("checkbox_select_multiple")
-        assert parse_form(form) == parse_expected("test_grouped_checkboxes.html")
+        if django.VERSION < (5, 2):
+            expected = "test_grouped_checkboxes_lt52.html"
+        else:
+            expected = "test_grouped_checkboxes.html"
+        assert parse_form(form) == parse_expected(expected)
         form.helper.layout = Layout("radio")
         assert parse_form(form) == parse_expected("test_grouped_radios.html")
 
@@ -595,6 +621,8 @@ class TestBootstrapLayoutObjects:
         form.helper.layout = Layout("checkbox_select_multiple")
         if django.VERSION < (5, 0):
             expected = "test_grouped_checkboxes_failing_lt50.html"
+        elif django.VERSION < (5, 2):
+            expected = "test_grouped_checkboxes_failing_lt52.html"
         else:
             expected = "test_grouped_checkboxes_failing.html"
         assert parse_form(form) == parse_expected(expected)
@@ -602,6 +630,8 @@ class TestBootstrapLayoutObjects:
         form.helper.layout = Layout("radio")
         if django.VERSION < (5, 0):
             expected = "test_grouped_radios_failing_lt50.html"
+        elif django.VERSION < (5, 2):
+            expected = "test_grouped_radios_failing_lt52.html"
         else:
             expected = "test_grouped_radios_failing.html"
         assert parse_form(form) == parse_expected(expected)
@@ -611,7 +641,11 @@ class TestBootstrapLayoutObjects:
         form["is_company"].help_text = "is_company help text"
         form.helper = FormHelper()
         form.helper.layout = Layout(Switch("is_company"))
-        assert parse_form(form) == parse_expected("test_switch.html")
+        if django.VERSION < (5, 2):
+            expected = "test_switch_lt52.html"
+        else:
+            expected = "test_switch.html"
+        assert parse_form(form) == parse_expected(expected)
 
     def test_switch_horizontal(self):
         form = SampleForm()
@@ -621,7 +655,11 @@ class TestBootstrapLayoutObjects:
         form.helper.field_class = "col-lg-8"
         form.helper.form_class = "form-horizontal"
         form.helper.layout = Layout(Switch("is_company"), "first_name")
-        assert parse_form(form) == parse_expected("test_switch_horizontal.html")
+        if django.VERSION < (5, 2):
+            expected = "test_switch_horizontal_lt52.html"
+        else:
+            expected = "test_switch_horizontal.html"
+        assert parse_form(form) == parse_expected(expected)
 
     def test_modal(self):
         test_form = SampleForm()
@@ -629,7 +667,7 @@ class TestBootstrapLayoutObjects:
         test_form.helper.form_tag = False
         test_form.helper.layout = Layout(
             Modal(
-                'field1',
+                "field1",
                 css_id="modal-id-ex",
                 css_class="modal-class-ex",
                 title="This is my modal",
@@ -637,8 +675,45 @@ class TestBootstrapLayoutObjects:
         )
         assert parse_form(test_form) == parse_expected("modal.html")
 
-    def test_inline_checkboxes(self):
-        form = CheckboxesSampleForm()
-        form.helper = FormHelper()
-        form.helper.layout = InlineRadios("checkboxes")
-        assert parse_form(form) == parse_expected("inline_checkboxes.html")
+    def test_formactions(self):
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.form_tag = False
+        test_form.helper.layout = Layout(
+            FormActions(
+                HTML("<b>test</b>"),
+            ),
+        )
+
+        assert 'class="mb-3  "' in render_crispy_form(test_form)
+
+    def test_formactions_attrs(self):
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.form_tag = False
+        test_form.helper.field_class = "field-class"
+        test_form.helper.layout = Layout(
+            FormActions(
+                HTML("<b>test</b>"),
+                css_class="formactions-test-class",
+                css_id="formactions-test-id",
+                test="formactions-test",
+            ),
+        )
+
+        assert parse_form(test_form) == parse_expected("test_formactions.html")
+
+    def test_formactions_horizontal_form(self):
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.form_tag = False
+        test_form.helper.form_class = "form-horizontal"
+        test_form.helper.layout = Layout(
+            FormActions(
+                HTML("<b>test</b>"),
+                css_class="formactions-test-class",
+            ),
+        )
+
+        expected_class = 'class="mb-3 row formactions-test-class "'
+        assert expected_class in render_crispy_form(test_form)
