@@ -35,7 +35,7 @@ def contains_partial(haystack, needle, ignore_needle_children=False):
 
 def parse_expected(expected_file):
     test_file = Path(TEST_DIR) / "results" / expected_file
-    with test_file.open() as f:
+    with open(test_file) as f:
         return parse_html(f.read())
 
 
